@@ -226,7 +226,7 @@ class TelegramBot(MultiBot[TelegramClient]):
 
             bytes_ = media.bytes_
             if inline_ and media.type_ is MediaType.AUDIO:
-                bytes_ = await flanautils.add_metadata(bytes_, {'title': 'bot_media.mp3'}, overwrite=False)
+                bytes_ = await flanautils.edit_metadata(bytes_, {'title': f'bot_media.{media.extension}'}, overwrite=False)
             file_ = io.BytesIO(bytes_)
             file_.name = f"{media.title or 'bot_media'}.{media.extension}"
             return file_
